'use strict';
const path = require('path');
const pkg = require('../package.json');
const log = require('@panther-cli-dev/log');
const semver = require('semver');
const colors = require('colors/safe');
const rootCheck = require('root-check');
const { LOWEST_NODE_VERSION, DEFAULT_CLI_HOME } = require('./const');
const userHome = require('user-home');
const pathExists = require('path-exists');
const minimist = require('minimist');
const { getNpmSemverVersion } = require('@panther-cli-dev/get-npm-info');

let args, config;
async function core() {
    try {
        checkPkgVersion()
        checkNodeVersion()
        checkRoot()
        checkUserHome()
        checkInputArgs()
        log.verbose('debug', 'test debug');
        checkEnv()
        await checkGlobalUpdate()
    } catch (error) {
        log.error(error.message)
    }
}

async function checkGlobalUpdate() {
    // 1. 获取当前版本号
    // 2. 调用npm API , 获取所有版本号
    // 3. 提取所有版本号, 对比那些版本号是大于当前版本号
    // 4. 获取最新的版本号, 提醒用户更新到该版本
    const currentVersion = pkg.version;
    const npmName = pkg.name;
    const lastVersion = await getNpmSemverVersion(currentVersion, npmName)
    if(lastVersion && semver.gt(lastVersion, currentVersion)) {
        log.warn('更新提示', colors.yellow(`请手动更新${npmName}, 当前版本是${currentVersion}, 最新版本是${lastVersion}, 更新命令: npm install -g ${npmName}`))
    }
    console.log("🚀 ~ file: index.js ~ line 38 ~ checkGlobalUpdate ~ versions", lastVersion)
}

function checkEnv() {
    const dotenv = require('dotenv');
    const dotenvPath = path.resolve(userHome, '.env')
    if(pathExists(dotenvPath)) {
        config = dotenv.config({
            path: dotenvPath
        });
    }
    config = createDefaultConfig()
    log.verbose('环境变量: CLI_HOME_PATH', process.env.CLI_HOME_PATH);
}

function createDefaultConfig() {
    const cliConfig = {
        home: userHome
    };
    if(process.env.CLI_HOME) {
        cliConfig['cliHome'] = path.join(userHome, process.env.CLI_HOME);
    } else {
        cliConfig['cliHome'] = path.join(userHome, DEFAULT_CLI_HOME);
    }
    process.env.CLI_HOME_PATH = cliConfig['cliHome']

}

function checkInputArgs() {
    args = minimist(process.argv.slice(2));
    checkArgs()
   console.log("🚀 ~ file: index.js ~ line 28 ~ checkInputArgs ~ args", args)
}

function checkArgs() {
    if(args.debug) {
        process.env.LOG_LEVEL = 'verbose';
    } else {
        process.env.LOG_LEVEL = 'info';
    }
    log.level = process.env.LOG_LEVEL
}

function checkPkgVersion() {
    log.notice('cli', pkg.version)
}

function checkNodeVersion() {
    // 获取node版本好
    // 比对最低版本号
    const currentNodeVersion = process.version;
    const lowestNodeVersion = LOWEST_NODE_VERSION
    if(!semver.gte(currentNodeVersion, lowestNodeVersion)) {
        throw new Error(colors.red(`panther-cli 需要安装 v${LOWEST_NODE_VERSION}以上版本的node`))
    }
}

function checkRoot() {
    rootCheck()
    console.log(process.getuid())
}

function checkUserHome() {
    console.log(userHome);
    const exists = pathExists.sync(userHome);
    if(!userHome || !exists) {
        throw new Error(colors.red('当前登录用户主目录不存在'))
    }
}

module.exports = core;
